@@ -47,18 +47,18 @@ typedef struct _EphyWebViewPrivate  EphyWebViewPrivate;
 
 typedef enum
 {
-	EPHY_WEB_VIEW_NAV_UP	= 1 << 0,
-	EPHY_WEB_VIEW_NAV_BACK	= 1 << 1,
+	EPHY_WEB_VIEW_NAV_UP		= 1 << 0,
+	EPHY_WEB_VIEW_NAV_BACK		= 1 << 1,
 	EPHY_WEB_VIEW_NAV_FORWARD	= 1 << 2
 } EphyWebViewNavigationFlags;
 
 typedef enum
 {
-	EPHY_WEB_VIEW_STATE_UNKNOWN	= 0,
+	EPHY_WEB_VIEW_STATE_UNKNOWN		= 0,
 	EPHY_WEB_VIEW_STATE_START		= 1 << 0,
-	EPHY_WEB_VIEW_STATE_REDIRECTING	= 1 << 1,
+	EPHY_WEB_VIEW_STATE_REDIRECTING		= 1 << 1,
 	EPHY_WEB_VIEW_STATE_TRANSFERRING	= 1 << 2,
-	EPHY_WEB_VIEW_STATE_NEGOTIATING	= 1 << 3,
+	EPHY_WEB_VIEW_STATE_NEGOTIATING		= 1 << 3,
 	EPHY_WEB_VIEW_STATE_STOP		= 1 << 4,
 
 	EPHY_WEB_VIEW_STATE_IS_REQUEST	= 1 << 5,
@@ -70,9 +70,9 @@ typedef enum
 
 typedef enum
 {
-	EPHY_WEB_VIEW_CHROME_MENUBAR	= 1 << 0,
-	EPHY_WEB_VIEW_CHROME_TOOLBAR	= 1 << 1,
-	EPHY_WEB_VIEW_CHROME_STATUSBAR	= 1 << 2,
+	EPHY_WEB_VIEW_CHROME_MENUBAR		= 1 << 0,
+	EPHY_WEB_VIEW_CHROME_TOOLBAR		= 1 << 1,
+	EPHY_WEB_VIEW_CHROME_STATUSBAR		= 1 << 2,
 	EPHY_WEB_VIEW_CHROME_BOOKMARKSBAR	= 1 << 3
 } EphyWebViewChrome;
 
@@ -121,8 +121,6 @@ struct _EphyWebViewClass
   WebKitWebViewClass parent_class;
 
   /* Signals */
-  void	 (* favicon)		(EphyWebView *view,
-                                 const char *location);
   void	 (* feed_link)		(EphyWebView *view,
                                  const char *type,
                                  const char *title,
@@ -145,8 +143,6 @@ struct _EphyWebViewClass
                                  const char *uri);
   gboolean (* modal_alert)	(EphyWebView *view);
   void	 (* modal_alert_closed) (EphyWebView *view);
-  void	 (* document_type)	(EphyWebView *view,
-                                 EphyWebViewDocumentType type);
   void	 (* new_window)		(EphyWebView *view,
                                  EphyWebView *new_view);
   gboolean (* search_key_press)	(EphyWebView *view,
@@ -178,8 +174,6 @@ const char *               ephy_web_view_get_link_message        (EphyWebView
 gboolean                   ephy_web_view_get_visibility          (EphyWebView                     *view);
 void                       ephy_web_view_set_link_message        (EphyWebView                     *view,
                                                                   char                            *link_message);
-void                       ephy_web_view_load_icon               (EphyWebView                     *view);
-void                       ephy_web_view_load_icon               (EphyWebView                     *view);
 void                       ephy_web_view_set_security_level      (EphyWebView                     *view,
                                                                   EphyWebViewSecurityLevel         level);
 void                       ephy_web_view_set_visibility          (EphyWebView                     *view,
@@ -206,16 +200,13 @@ void                       ephy_web_view_print_preview_navigate  (EphyWebView
 GSList *                   ephy_web_view_get_go_up_list          (EphyWebView                     *view);
 void                       ephy_web_view_set_title               (EphyWebView                     *view,
                                                                   const char                      *view_title);
-const char *               ephy_web_view_get_icon_address        (EphyWebView                     *view);
 const char *               ephy_web_view_get_title               (EphyWebView                     *view);
 gboolean                   ephy_web_view_can_go_up               (EphyWebView                     *view);
 const char *               ephy_web_view_get_address             (EphyWebView                     *view);
 const char *               ephy_web_view_get_title_composite     (EphyWebView                     *view);
 
 /* These should be private */
-void                       ephy_web_view_set_address             (EphyWebView                     *embed,
-                                                                  const char                      *address);
-void                       ephy_web_view_set_icon_address        (EphyWebView                     *view,
+void                       ephy_web_view_set_address             (EphyWebView                     *view,
                                                                   const char                      *address);
 void                       ephy_web_view_update_from_net_state   (EphyWebView                     *view,
                                                                   const char                      *uri,
