@@ -39,6 +39,14 @@
 
 #include <string.h>
 
+/**
+ * SECTION:ephy-location-entry
+ * @short_description: A location entry widget
+ * @see_also: #GtkEntry
+ *
+ * #EphyLocationEntry implements the location bar in the main Epiphany window.
+ */
+
 #define EPHY_LOCATION_ENTRY_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_LOCATION_ENTRY, EphyLocationEntryPrivate))
 
 struct _EphyLocationEntryPrivate
@@ -211,6 +219,13 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 
 	widget_class->style_set = ephy_location_entry_style_set;
 
+    /**
+	* EphyLocationEntry::user-changed:
+	* @entry: the object on which the signal is emitted
+	*
+	* Emitted when the user changes the contents of the internal #GtkEntry
+	*
+	*/
 	signals[USER_CHANGED] = g_signal_new (
 		"user_changed", G_OBJECT_CLASS_TYPE (klass),
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
@@ -221,6 +236,14 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		0,
 		G_TYPE_NONE);
 
+    /**
+	* EphyLocationEntry::lock-clicked:
+	* @entry: the object on which the signal is emitted
+	*
+	* Emitted when the user clicks the security icon inside the
+	* #EphyLocationEntry.
+	*
+	*/
 	signals[LOCK_CLICKED] = g_signal_new (
 		"lock-clicked",
 		EPHY_TYPE_LOCATION_ENTRY,
@@ -231,6 +254,16 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		G_TYPE_NONE,
 		0);
 
+    /**
+	* EphyLocationEntry::get-location:
+	* @entry: the object on which the signal is emitted
+	* Returns: the current page address as a string
+	*
+	* For drag and drop purposes, the location bar will request you the real
+	* address of where it is pointing to. The signal handler for this function
+	* should return the address of the currently loaded site.
+	*
+	*/
 	signals[GET_LOCATION] = g_signal_new (
 		"get-location", G_OBJECT_CLASS_TYPE (klass),
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
@@ -241,6 +274,16 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		0,
 		G_TYPE_NONE);
 
+    /**
+	* EphyLocationEntry::get-title:
+	* @entry: the object on which the signal is emitted
+	* Returns: the current page title as a string
+	*
+	* For drag and drop purposes, the location bar will request you the title
+	* of where it is pointing to. The signal handler for this function
+	* should return the title of the currently loaded site.
+	*
+	*/
 	signals[GET_TITLE] = g_signal_new (
 		"get-title", G_OBJECT_CLASS_TYPE (klass),
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
@@ -518,7 +561,7 @@ match_selected_cb (GtkEntryCompletion *completion,
 	priv->needs_reset = (state == GDK_CONTROL_MASK ||
 			     state == (GDK_CONTROL_MASK | GDK_SHIFT_MASK));
 
-	ephy_location_entry_set_location (entry, item, NULL);
+	ephy_location_entry_set_location (entry, item);
 	//gtk_im_context_reset (GTK_ENTRY (entry->priv->entry)->im_context);
 	g_signal_emit_by_name (priv->entry, "activate");
 
@@ -1261,18 +1304,13 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 /**
  * ephy_location_entry_set_location:
  * @entry: an #EphyLocationEntry widget
- * @address: the address to be set as the current location
- * @typed_address: the address to be shown in the location entry or NULL
- *
- * Sets the current address of @entry to @address, alternatively a
- * @typed_address can be used to determine the text to be displayed in the
- * location entry.
+ * @address: new location address
  *
+ * Sets the current address of @entry to @address.
  **/
 void
 ephy_location_entry_set_location (EphyLocationEntry *entry,
-				  const char *address,
-				  const char *typed_address)
+				  const char *address)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkClipboard *clipboard;
@@ -1280,8 +1318,6 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	char* selection = NULL;
 	int start, end;
 
-	g_return_if_fail (address != NULL);
-
 	/* Setting a new text will clear the clipboard. This makes it impossible
 	 * to copy&paste from the location entry of one tab into another tab, see
 	 * bug #155824. So we save the selection iff the clipboard was owned by
@@ -1302,11 +1338,7 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 		}
 	}
 
-	if (typed_address != NULL)
-	{
-		text = typed_address;
-	}
-	else if (address != NULL && strcmp (address, "about:blank") != 0)
+	if (address != NULL && strcmp (address, "about:blank") != 0)
 	{
 		text = address;
 	}
@@ -1316,7 +1348,7 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	}
 
 	/* First record the new hash, then update the entry text */
-	priv->hash = g_str_hash (address);
+	priv->hash = g_str_hash (text);
 
 	priv->block_update = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (priv->entry), text);
@@ -1412,7 +1444,7 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 
 	retval = g_str_hash (text) != g_str_hash (old_text);
 
-	ephy_location_entry_set_location (entry, text, NULL);
+	ephy_location_entry_set_location (entry, text);
 	g_free (url);
 
 	if (notify)
