@@ -445,7 +445,6 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
 
 	in_new_window = in_new_window && !eel_gconf_get_boolean (CONF_LOCKDOWN_FULLSCREEN);
-	g_return_val_if_fail (in_new_window || !is_popup, NULL);
 	g_return_val_if_fail (open_page == (gboolean)(request != NULL), NULL);
 
 	jump_to = (flags & EPHY_NEW_TAB_JUMP) != 0;
@@ -467,7 +466,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_embed != NULL)
 	{
 		nb = ephy_window_get_notebook (window);
-                /* FIXME this assumes the tab is the direct notebook child */
+		/* FIXME this assumes the tab is the direct notebook child */
 		position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
 						  GTK_WIDGET (previous_embed)) + 1;
 	}
@@ -478,8 +477,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		if (embed != NULL)
 		{
-			if (ephy_web_view_get_is_blank (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)) &&
-			    ephy_web_view_is_loading (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)) == FALSE)
+			if (ephy_web_view_get_is_blank (ephy_embed_get_web_view (embed)) &&
+			    ephy_web_view_is_loading (ephy_embed_get_web_view (embed)) == FALSE)
 			{
 				active_is_blank = TRUE;
 			}
@@ -496,8 +495,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 
 	if (previous_embed != NULL)
 	{	
-                ephy_web_view_copy_back_history (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (previous_embed),
-                                                 EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+		ephy_web_view_copy_back_history (ephy_embed_get_web_view (previous_embed),
+						 ephy_embed_get_web_view (embed));
 	}		
 
 	ephy_gui_window_update_user_time (GTK_WIDGET (window), user_time);
@@ -515,38 +514,38 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
 	    flags & EPHY_NEW_TAB_NEW_PAGE)
 	{
-		ephy_web_view_set_typed_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), "");
+		ephy_web_view_set_typed_address (ephy_embed_get_web_view (embed), "");
 		ephy_toolbar_activate_location (toolbar);
 		is_empty = load_homepage (embed);
 	}
 	else if (flags & EPHY_NEW_TAB_OPEN_PAGE)
 	{
-                ephy_web_view_load_request (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed),
-                                            request);
+		ephy_web_view_load_request (ephy_embed_get_web_view (embed),
+					    request);
 
 		is_empty = url_is_empty (webkit_network_request_get_uri (request));
 	}
 
-        /* Make sure the initial focus is somewhere sensible and not, for
-         * example, on the reload button.
-         */
-        if (in_new_window || jump_to)
-        {
-                /* If the location entry is blank, focus that, except if the
-                 * page was a copy */
-                if (is_empty)
-                {
-                        /* empty page, focus location entry */
+	/* Make sure the initial focus is somewhere sensible and not, for
+	 * example, on the reload button.
+	 */
+	if (in_new_window || jump_to)
+	{
+		/* If the location entry is blank, focus that, except if the
+		 * page was a copy */
+		if (is_empty)
+		{
+			/* empty page, focus location entry */
 			toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
 			ephy_toolbar_activate_location (toolbar);
-                }
-                else if (embed != NULL) 
-                {
+		}
+		else if (embed != NULL)
+		{
 			/* non-empty page, focus the page. but make sure the widget is realised first! */
 			gtk_widget_realize (GTK_WIDGET (embed));
 			gtk_widget_grab_focus (GTK_WIDGET (embed));
-                }
-        }
+		}
+	}
 
 	return embed;
 }
