@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2000-2004 Marco Pesenti Gritti
+ *  Copyright © 2009 Collabora Ltd.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +21,7 @@
 
 #include "config.h"
 
+#include "ephy-embed.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
@@ -27,7 +29,6 @@
 #include "ephy-shell.h"
 #include "ephy-embed-persist.h"
 #include "ephy-debug.h"
-#include "ephy-command-manager.h"
 #include "window-commands.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
@@ -56,6 +57,7 @@
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <webkit/webkit.h>
 
 static void
 page_setup_done_cb (GtkPageSetup *setup,
@@ -92,7 +94,7 @@ window_cmd_file_print_preview (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	ephy_web_view_set_print_preview_mode (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), TRUE);
+	ephy_web_view_set_print_preview_mode (ephy_embed_get_web_view (embed), TRUE);
 	_ephy_window_set_print_preview (window, TRUE);
 }
 
@@ -123,8 +125,8 @@ window_cmd_file_send_to	(GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	location = ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
-	title = ephy_web_view_get_title (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	location = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
+	title = ephy_web_view_get_title (ephy_embed_get_web_view (embed));
 
 	subject = g_uri_escape_string (title, NULL, TRUE);
 	body = g_uri_escape_string (location, NULL, TRUE);
@@ -246,8 +248,8 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	g_return_if_fail (embed != NULL);
 
 	ephy_bookmarks_ui_add_bookmark (GTK_WINDOW (window),
-					ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)),
-					ephy_web_view_get_title (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)));
+					ephy_web_view_get_address (ephy_embed_get_web_view (embed)),
+					ephy_web_view_get_title (ephy_embed_get_web_view (embed)));
 }
 
 static void
@@ -275,6 +277,32 @@ open_response_cb (GtkDialog *dialog, int response, EphyWindow *window)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
+static void
+save_response_cb (GtkDialog *dialog, int response, EphyEmbed *embed)
+{
+	if (response == GTK_RESPONSE_ACCEPT)
+	{
+		char *uri, *converted;
+
+		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+		if (uri != NULL)
+		{
+			converted = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
+
+			if (converted != NULL)
+			{
+				EphyWebView *web_view = ephy_embed_get_web_view (embed);
+				ephy_web_view_save (web_view, converted);
+			}
+
+			g_free (converted);
+			g_free (uri);
+	        }
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 void
 window_cmd_file_open (GtkAction *action,
 		      EphyWindow *window)
@@ -298,26 +326,21 @@ window_cmd_file_save_as (GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyEmbed *embed;
-	EphyEmbedPersist *persist;
+	EphyFileChooser *dialog;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	persist = EPHY_EMBED_PERSIST
-		(g_object_new (EPHY_TYPE_EMBED_PERSIST, NULL));
+	dialog = ephy_file_chooser_new (_("Save"),
+					GTK_WIDGET (window),
+					GTK_FILE_CHOOSER_ACTION_SAVE,
+					CONF_STATE_SAVE_DIR,
+					EPHY_FILE_FILTER_ALL_SUPPORTED);
 
-	ephy_embed_persist_set_embed (persist, embed);
-	ephy_embed_persist_set_fc_title (persist, _("Save As"));
-	ephy_embed_persist_set_fc_parent (persist,GTK_WINDOW (window));
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (save_response_cb), embed);
 
-	ephy_embed_persist_set_flags
-		(persist, EPHY_EMBED_PERSIST_MAINDOC | EPHY_EMBED_PERSIST_ASK_DESTINATION);
-	ephy_embed_persist_set_persist_key
-		(persist, CONF_STATE_SAVE_DIR);
-
-	ephy_embed_persist_save (persist);
-
-	g_object_unref (G_OBJECT(persist));
+	gtk_widget_show (GTK_WIDGET (dialog));
 }
 
 void
@@ -374,8 +397,7 @@ window_cmd_edit_undo (GtkAction *action,
 
 		if (embed)
 		{
-			ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
-							 "cmd_undo");
+			webkit_web_view_undo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (EPHY_EMBED (embed)));
 		}
 	}
 }
@@ -400,8 +422,7 @@ window_cmd_edit_redo (GtkAction *action,
 		embed = gtk_widget_get_ancestor (widget, EPHY_TYPE_EMBED);
 		if (embed)
 		{
-			ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
-							 "cmd_redo");
+			webkit_web_view_redo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (EPHY_EMBED (embed)));
 		}
 	}
 }
@@ -421,8 +442,7 @@ window_cmd_edit_cut (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_cut");
+		webkit_web_view_cut_clipboard (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
@@ -443,8 +463,7 @@ window_cmd_edit_copy (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_copy");
+		webkit_web_view_copy_clipboard (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
@@ -465,8 +484,7 @@ window_cmd_edit_paste (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_paste");
+		webkit_web_view_paste_clipboard (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
@@ -487,8 +505,11 @@ window_cmd_edit_delete (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
+		/* FIXME: TODO */
+#if 0
 		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
 						 "cmd_delete");
+#endif
 	}
 }
 
@@ -510,8 +531,7 @@ window_cmd_edit_select_all (GtkAction *action,
                   (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_selectAll");
+		webkit_web_view_select_all (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
@@ -595,29 +615,151 @@ window_cmd_view_zoom_normal (GtkAction *action,
 }
 
 static void
-save_source_completed_cb (EphyEmbedPersist *persist)
+save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointer data)
 {
-	const char *dest;
-	guint32 user_time;
+	char *uri;
 	GFile *file;
+	GError *error = NULL;
 
-	user_time = ephy_embed_persist_get_user_time (persist);
-	dest = ephy_embed_persist_get_dest (persist);
-	g_return_if_fail (dest != NULL);
+	g_output_stream_close_finish (ostream, result, &error);
+	if (error)
+	{
+		g_warning ("Unable to close file: %s", error->message);
+		g_error_free (error);
+		return;
+	}
 
-	file = g_file_new_for_path (dest);
-	ephy_file_delete_on_exit (file);
+	uri = (char*)g_object_get_data (G_OBJECT (ostream), "ephy-save-temp-source-uri");
 
-	ephy_file_launch_handler ("text/plain", file, user_time);
+	file = g_file_new_for_uri (uri);
+	if (!ephy_file_launch_handler ("text/plain", file, gtk_get_current_event_time ()))
+	{
+		/* Fallback to view the source inside the browser */
+		const char *uri;
+		EphyEmbed *embed, *new_embed;
+
+		uri = (const char*) g_object_get_data (G_OBJECT (ostream),
+						       "ephy-original-source-uri");
+		embed = (EphyEmbed*)g_object_get_data (G_OBJECT (ostream),
+						       "ephy-save-temp-source-embed");
+
+		new_embed = ephy_shell_new_tab (ephy_shell_get_default (),
+						EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
+						embed,
+						NULL,
+						EPHY_NEW_TAB_JUMP | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+
+		webkit_web_view_set_view_source_mode (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_embed),
+						      TRUE);
+		webkit_web_view_load_uri (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_embed),
+					  uri);
+	}
+
+	g_object_unref (ostream);
 	g_object_unref (file);
 }
 
+static void
+save_temp_source_write_cb (GOutputStream *ostream, GAsyncResult *result, GString *data)
+{
+	GError *error = NULL;
+	gssize written;
+
+	written = g_output_stream_write_finish (ostream, result, &error);
+	if (error)
+	{
+		g_string_free (data, TRUE);
+		g_warning ("Unable to write to file: %s", error->message);
+		g_error_free (error);
+
+		g_output_stream_close_async (ostream, G_PRIORITY_DEFAULT, NULL,
+					     (GAsyncReadyCallback)save_temp_source_close_cb,
+					     NULL);
+
+		return;
+	}
+
+	if (written == data->len)
+	{
+		g_string_free (data, TRUE);
+
+		g_output_stream_close_async (ostream, G_PRIORITY_DEFAULT, NULL,
+					     (GAsyncReadyCallback)save_temp_source_close_cb,
+					     NULL);
+
+		return;
+	}
+
+	data->len -= written;
+	data->str += written;
+
+	g_output_stream_write_async (ostream,
+				     data->str, data->len,
+				     G_PRIORITY_DEFAULT, NULL,
+				     (GAsyncReadyCallback)save_temp_source_write_cb,
+				     data);
+}
+
+static void
+save_temp_source_replace_cb (GFile *file, GAsyncResult *result, EphyEmbed *embed)
+{
+	EphyWebView *view;
+	WebKitWebFrame *frame;
+	WebKitWebDataSource *data_source;
+	GString *const_data;
+	GString *data;
+	GFileOutputStream *ostream;
+	GError *error = NULL;
+
+	ostream = g_file_replace_finish (file, result, &error);
+	if (error)
+	{
+		g_warning ("Unable to replace file: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	g_object_set_data_full (G_OBJECT (ostream),
+				"ephy-save-temp-source-uri",
+				g_file_get_uri (file),
+				g_free);
+
+	view = ephy_embed_get_web_view (embed);
+
+	g_object_set_data_full (G_OBJECT (ostream),
+				"ephy-original-source-uri",
+				g_strdup (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (view))),
+				g_free),
+
+	g_object_set_data_full (G_OBJECT (ostream),
+				"ephy-save-temp-source-embed",
+				g_object_ref (embed),
+				g_object_unref);
+
+	frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (view));
+	data_source = webkit_web_frame_get_data_source (frame);
+	const_data = webkit_web_data_source_get_data (data_source);
+
+	/* We create a new GString here because we need to make sure
+	 * we keep writing in case of partial writes */
+	if (const_data)
+		data = g_string_new_len (const_data->str, const_data->len);
+	else
+		data = g_string_new_len ("", 0);
+
+	g_output_stream_write_async (G_OUTPUT_STREAM (ostream),
+				     data->str, data->len,
+				     G_PRIORITY_DEFAULT, NULL,
+				     (GAsyncReadyCallback)save_temp_source_write_cb,
+				     data);
+}
+
 static void
 save_temp_source (EphyEmbed *embed,
 		  guint32 user_time)
 {
+	GFile *file;
 	char *tmp, *base;
-	EphyEmbedPersist *persist;
 	const char *static_temp_dir;
 
 	static_temp_dir = ephy_file_tmp_dir ();
@@ -634,21 +776,14 @@ save_temp_source (EphyEmbed *embed,
 		return;
 	}
 
-	persist = EPHY_EMBED_PERSIST
-		(g_object_new (EPHY_TYPE_EMBED_PERSIST, NULL));
-
-	ephy_embed_persist_set_embed (persist, embed);
-	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_COPY_PAGE |
-				      EPHY_EMBED_PERSIST_NO_VIEW);
-	ephy_embed_persist_set_dest (persist, tmp);
-	ephy_embed_persist_set_user_time (persist, user_time);
-
-	g_signal_connect (persist, "completed",
-			  G_CALLBACK (save_source_completed_cb), NULL);
-
-	ephy_embed_persist_save (persist);
-	g_object_unref (persist);
+	file = g_file_new_for_path (tmp);
+	g_file_replace_async (file, NULL, FALSE,
+			      G_FILE_CREATE_REPLACE_DESTINATION|G_FILE_CREATE_PRIVATE,
+			      G_PRIORITY_DEFAULT, NULL,
+			      (GAsyncReadyCallback)save_temp_source_replace_cb,
+			      embed);
 
+	g_object_unref (file);
 	g_free (tmp);
 }
 
@@ -664,7 +799,7 @@ window_cmd_view_page_source (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	address = ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 	user_time = gtk_get_current_event_time ();
 
 	if (g_str_has_prefix (address, "file://"))
@@ -692,7 +827,7 @@ window_cmd_view_page_security_info (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	ephy_web_view_show_page_certificate (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	ephy_web_view_show_page_certificate (ephy_embed_get_web_view (embed));
 }
 
 void
@@ -720,7 +855,7 @@ window_cmd_edit_personal_data (GtkAction *action,
           (EPHY_EMBED_CONTAINER (window));
 	if (embed == NULL) return;
 
-	address = ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
+	address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 	
 	host = address != NULL ? ephy_string_get_host_name (address) : NULL;
 
@@ -760,7 +895,7 @@ void
 window_cmd_help_contents (GtkAction *action,
 			 EphyWindow *window)
 {
-	ephy_gui_help (GTK_WINDOW (window), "epiphany", NULL);
+	ephy_gui_help (GTK_WIDGET (window), NULL);
 }
 
 #define ABOUT_GROUP "About"
